import hashlib
import re
import os
import random
import string
import subprocess
import base64
from urllib.parse import urlparse


class Custombytes(bytes):
    def contains(self, a, encoding="utf-8"):
        if isinstance(a, str):
            return a in self.decode(encoding)
        elif isinstance(a, bytes):
            return a in self
        else:
            raise SyntaxError("`contains` method argument must be str or bytes")

    def match(self, a, encoding="utf-8"):
        if isinstance(a, str):
            pattern = re.compile(a)
            ret = re.search(pattern, self.decode(encoding))
        elif isinstance(a, bytes):
            pattern = re.compile(a)
            ret = re.search(pattern, self)
        else:
            raise SyntaxError("`match` method argument must be str or bytes")
        return ret if ret else False


class Customstr(str):
    def contains(self, a, encoding="utf-8"):
        if isinstance(a, str):
            return a in self
        elif isinstance(a, bytes):
            return a in self.encode(encoding)
        else:
            raise SyntaxError("`contains` method argument must be str or bytes")

    def match(self, a, encoding="utf-8"):
        if isinstance(a, str):
            pattern = re.compile(a)
            ret = re.search(pattern, self)
        elif isinstance(a, bytes):
            pattern = re.compile(a)
            ret = re.search(pattern, self.encode(encoding))
        else:
            raise SyntaxError("`match` method argument must be str or bytes")
        return ret if ret else False


def _add_extra_method(s):
    if isinstance(s, str):
        return Customstr(s)
    elif isinstance(s, bytes):
        return Custombytes(s)
    else:
        return s


def md5(x):
    if isinstance(x, str):
        x = x.encode("utf-8")
    return hashlib.md5(x).hexdigest()


def rand(length, _type="all"):
    if _type == "lowercase":
        s = string.ascii_lowercase
    elif _type == "uppercase":
        s = string.ascii_uppercase
    elif _type == "int":
        s = string.digits
    elif _type == "all":
        s = string.ascii_letters + string.digits
    else:
        raise SyntaxError("type must be `lowercase, uppercase, int, all`")
    return ''.join(random.sample(s, length))


def randint(x, y):
    return random.randint(x, y)


def convert_bytes(s, encoding="utf-8"):
    if not isinstance(s, str):
        s = str(s)
    return s.encode(encoding)


def b64encode(s):
    if isinstance(s, str):
        s = s.encode("utf-8")
    return base64.b64encode(s)


def b64decode(s):
    return base64.b64decode(s)


def invoke_jar(jar_path, *args, proxies=None, timeout=120):
    """
    :param jar_path: java程序路径
    :param args: java参数
    :param proxies: 是否使用代理{"http:": "", "https:": ""}
    :param timeout: 默认超时时间
    :return:
    """
    if not os.path.isfile(jar_path):
        jar_path = os.path.join("plugins", "poc_yml", jar_path)
    cmd = ["java", "-jar", jar_path, *args]
    if proxies:
        if proxies.get("https_proxy"):
            host, port = parse_url(proxies.get("https_proxy"))
            cmd.append(f"-Dhttps.proxyHost={host}", f"-Dhttps.proxyPort={port}")
        if proxies.get("http_proxy"):
            host, port = parse_url(proxies.get("http_proxy"))
            cmd.append(f"-Dhttp.proxyHost={host}", f"-Dhttp.proxyPort={port}")
    try:
        proc = subprocess.run(cmd, timeout=timeout, capture_output=True)
    except subprocess.TimeoutExpired:
        return Customstr("")
    except FileNotFoundError:
        return Customstr("`Cannont found java in PATH, please install jdk first`")
    if proc.returncode == 0:
        return Custombytes(proc.stdout)
    else:
        return Custombytes(proc.stderr)


def parse_url(url):
    parse_result = urlparse(url)
    try:
        host, port = parse_result.netloc.split(":")
        return host, port
    except ValueError:
        if parse_result.scheme == "http":
            port = 80
        elif parse_result.scheme == "https":
            port = 443
        else:
            port = 1080
        return parse_result.netloc, port
